@@ -1,76 +1,18 @@
 # -*- coding: utf-8 -*-
-"""
-###############################################################################
-# Copyright 2012 Grigoriy Kramarenko.
-###############################################################################
-# This file is part of QUICKAPI.
-#
-#    QUICKAPI is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#    (at your option) any later version.
-#
-#    QUICKAPI is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
-#
-#    You should have received a copy of the GNU General Public License
-#    along with QUICKAPI.  If not, see <http://www.gnu.org/licenses/>.
-#
-# Этот файл — часть QUICKAPI.
-#
-#   QUICKAPI - свободная программа: вы можете перераспространять ее и/или
-#   изменять ее на условиях Стандартной общественной лицензии GNU в том виде,
-#   в каком она была опубликована Фондом свободного программного обеспечения;
-#   либо версии 3 лицензии, либо (по вашему выбору) любой более поздней
-#   версии.
-#
-#   QUICKAPI распространяется в надежде, что она будет полезной,
-#   но БЕЗО ВСЯКИХ ГАРАНТИЙ; даже без неявной гарантии ТОВАРНОГО ВИДА
-#   или ПРИГОДНОСТИ ДЛЯ ОПРЕДЕЛЕННЫХ ЦЕЛЕЙ. Подробнее см. в Стандартной
-#   общественной лицензии GNU.
-#
-#   Вы должны были получить копию Стандартной общественной лицензии GNU
-#   вместе с этой программой. Если это не так, см.
-#   <http://www.gnu.org/licenses/>.
-###############################################################################
-
-Структура ответа outdict = {
-    'status': 200, # коды HTTP, имеющие смысл.
-    'message': u'Строчное сообщение для пользователя',
-    'data': <Сериализованный объект JSON>,
-}
-
-Пример outdict при возврате единичной настройки пользователя:
-outdic = {
-    'status': 200,
-    'message': u'Количество объектов на одной странице',
-    'data': 25,
-}
-
-Пример outdict с переадресацией:
-outdic = {
-    'status': 303,
-    'message': u'Смотрите на другой странице',
-    'data': { 'Location': '/other-page/', },
-},
-либо:
-outdict = {
-    'status': 401,
-    'message': u'Пользователь не авторизован',
-    'data': { 'Location': '/accounts/login/', },
-}
-"""
+from __future__ import unicode_literals
+from django.utils.encoding import smart_text
+from django.utils import six
 from django.conf import settings
 from django.http import HttpResponse
 from django.utils.functional import Promise
 from django.utils.translation import ugettext_lazy as _
 from django.utils.timezone import is_aware
+from django.utils.formats import number_format
 
-from quickapi.conf import QUICKAPI_INDENT
+from quickapi.conf import (QUICKAPI_INDENT, QUICKAPI_DECIMAL_LOCALE,
+    QUICKAPI_ENSURE_ASCII)
 
-import datetime, decimal, json as jsonlib
+import datetime, decimal, zlib, json as jsonlib
 
 MESSAGES = {
 #1xx
@@ -159,32 +101,35 @@ class JSONEncoder(jsonlib.JSONEncoder):
                 r = r[:12]
             return r
         elif isinstance(o, decimal.Decimal):
-            return float(o)
+            if QUICKAPI_DECIMAL_LOCALE:
+                return number_format(o, use_l10n=True, force_grouping=True)
+            else:
+                return str(o)
         elif isinstance(o, Promise):
-            return unicode(o)
+            return six.text_type(o)
         else:
             return super(JSONEncoder, self).default(o)
 
 DjangoJSONEncoder = JSONEncoder
 
-def _get_json_response(ctx=None):
-    result = jsonlib.dumps(ctx, ensure_ascii=True, 
-                            cls=DjangoJSONEncoder,
-                            indent=QUICKAPI_INDENT,
-                        )
-    try:
-        result = result.encode(settings.DEFAULT_CHARSET)
-    except:
-        pass
-    content_type = "%s; charset=%s" % ("application/json",
-                    settings.DEFAULT_CHARSET)
+def tojson(ctx, indent=None):
+    result = jsonlib.dumps(ctx, ensure_ascii=QUICKAPI_ENSURE_ASCII, cls=JSONEncoder, indent=indent)
+    result = result.encode(settings.DEFAULT_CHARSET)
+    return result
+
+def get_json_response(ctx=None):
+    result = tojson(ctx, indent=QUICKAPI_INDENT)
+    content_type = "%s; charset=%s" % ("application/json", settings.DEFAULT_CHARSET)
     response = HttpResponse(content_type=content_type)
     if len(result)>512:
         response['Content-encoding'] = 'deflate'
-        result = result.encode('zlib')
+        result = zlib.compress(result)
     response.write(result)
     return response
 
+# compatibility older
+_get_json_response = get_json_response
+
 def check_status(dic):
     if not isinstance(dic['status'], int):
         dic['status'] = int(dic['status'])
@@ -203,7 +148,7 @@ def JSONResponse(data=None, message=None, status=200, **kwargs):
     }
     dic.update(kwargs)
     dic = check_message(check_status(dic))
-    return _get_json_response(dic)
+    return get_json_response(dic)
 
 def JSONRedirect(location='/', message=None, status=301, **kwargs):
     dic = {
@@ -213,4 +158,4 @@ def JSONRedirect(location='/', message=None, status=301, **kwargs):
     }
     dic.update(kwargs)
     dic = check_message(check_status(dic))
-    return _get_json_response(dic)
+    return get_json_response(dic)
