@@ -1,41 +1,7 @@
 # -*- coding: utf-8 -*-
-"""
-###############################################################################
-# Copyright 2012 Grigoriy Kramarenko.
-###############################################################################
-# This file is part of QUICKAPI.
-#
-#    QUICKAPI is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#    (at your option) any later version.
-#
-#    QUICKAPI is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
-#
-#    You should have received a copy of the GNU General Public License
-#    along with QUICKAPI.  If not, see <http://www.gnu.org/licenses/>.
-#
-# Этот файл — часть QUICKAPI.
-#
-#   QUICKAPI - свободная программа: вы можете перераспространять ее и/или
-#   изменять ее на условиях Стандартной общественной лицензии GNU в том виде,
-#   в каком она была опубликована Фондом свободного программного обеспечения;
-#   либо версии 3 лицензии, либо (по вашему выбору) любой более поздней
-#   версии.
-#
-#   QUICKAPI распространяется в надежде, что она будет полезной,
-#   но БЕЗО ВСЯКИХ ГАРАНТИЙ; даже без неявной гарантии ТОВАРНОГО ВИДА
-#   или ПРИГОДНОСТИ ДЛЯ ОПРЕДЕЛЕННЫХ ЦЕЛЕЙ. Подробнее см. в Стандартной
-#   общественной лицензии GNU.
-#
-#   Вы должны были получить копию Стандартной общественной лицензии GNU
-#   вместе с этой программой. Если это не так, см.
-#   <http://www.gnu.org/licenses/>.
-###############################################################################
-"""
+from __future__ import unicode_literals, print_function
+from django.utils.encoding import smart_text
+from django.utils import six
 from django.contrib.auth import authenticate, login
 from django.core.mail import mail_admins
 from django.shortcuts import render_to_response
@@ -46,12 +12,18 @@ from django.utils.termcolors import colorize
 from django.utils import translation
 from django.utils.translation import ugettext_lazy as _
 
-from quickapi.http import JSONResponse, MESSAGES
-from quickapi.conf import (settings, QUICKAPI_DEFINED_METHODS,
-    QUICKAPI_ONLY_AUTHORIZED_USERS, QUICKAPI_DEBUG, DEBUG, SITE_ID,
-    QUICKAPI_SWITCH_LANGUAGE, QUICKAPI_SWITCH_LANGUAGE_AUTO)
+from quickapi.http import JSONResponse, JSONRedirect, MESSAGES
+from quickapi.conf import (settings, DEBUG, SITE_ID,
+    QUICKAPI_DEFINED_METHODS,
+    QUICKAPI_ONLY_AUTHORIZED_USERS,
+    QUICKAPI_INDENT,
+    QUICKAPI_DEBUG,
+    QUICKAPI_SWITCH_LANGUAGE,
+    QUICKAPI_SWITCH_LANGUAGE_AUTO,
+    QUICKAPI_DECIMAL_LOCALE,
+    QUICKAPI_ENSURE_ASCII)
 
-import traceback, json as jsonlib, decimal
+import traceback, decimal, json as jsonlib, decimal
 
 if 'django.contrib.sites' in settings.INSTALLED_APPS:
     from django.contrib.sites.models import Site
@@ -64,12 +36,12 @@ def switch_language(request, code=None):
     Переключает язык для приложения, если такое переключение не
     запрещено в настройках.
     """
-    old_language = translation.get_language()
 
     if not QUICKAPI_SWITCH_LANGUAGE and not code:
         # Disabled switching for quickapi only
-        return settings.LANGUAGE_CODE, old_language
+        return settings.LANGUAGE_CODE, None
 
+    old_language = translation.get_language()
     new_language = None
 
     if code:
@@ -81,7 +53,7 @@ def switch_language(request, code=None):
 
     if new_language:
         try:
-            new_language = translation.activate(new_language)
+            translation.activate(new_language)
         except:
             translation.activate(old_language)
             new_language = None
@@ -89,22 +61,49 @@ def switch_language(request, code=None):
     return old_language, new_language
 
 @csrf_exempt
-def test(request):
+def test(request, code=200, redirect='/'):
     """
     Test response
     """
 
-    default_language, request_language = switch_language(request)
+    if code in (301, 302):
+        return JSONRedirect(location=redirect, status=code,
+                message=_('Test response. Redirect to %s.') % redirect)
+    elif code == 400:
+        return JSONResponse(status=400, message=_('Test response. Error 400.'))
+    elif code == 500:
+        return JSONResponse(status=500, message=_('Test response. Error 500.'))
+
+    now = timezone.now()
 
     data = {
         'REMOTE_ADDR': request.META.get("HTTP_X_REAL_IP", request.META.get("REMOTE_ADDR", None)),
         'REMOTE_HOST': request.META.get("REMOTE_HOST", None),
-        'default language': default_language,
-        'request language': request_language,
-        'string': _('String in your localization'),
-        'datetime': timezone.now(),
+        'default language': settings.LANGUAGE_CODE,
+        'request language': request.POST.get('language', getattr(request, 'LANGUAGE_CODE', None)),
         'is_authenticated': request.user.is_authenticated(),
+        'types': {
+            'string': _('String in your localization'),
+            'datetime': now,
+            'date': now.date(),
+            'time': now.time(),
+            'decimal': decimal.Decimal('12345678.90'),
+            'float': 12345678.90,
+            'integer': 1234567890,
+        },
+        'settings': _('Hidden in not debug mode'),
     }
+    if settings.DEBUG:
+        data['settings'] = {
+            'QUICKAPI_DEFINED_METHODS': QUICKAPI_DEFINED_METHODS,
+            'QUICKAPI_ONLY_AUTHORIZED_USERS': QUICKAPI_ONLY_AUTHORIZED_USERS,
+            'QUICKAPI_INDENT': QUICKAPI_INDENT,
+            'QUICKAPI_DEBUG': QUICKAPI_DEBUG,
+            'QUICKAPI_SWITCH_LANGUAGE': QUICKAPI_SWITCH_LANGUAGE,
+            'QUICKAPI_SWITCH_LANGUAGE_AUTO': QUICKAPI_SWITCH_LANGUAGE_AUTO,
+            'QUICKAPI_DECIMAL_LOCALE': QUICKAPI_DECIMAL_LOCALE,
+            'QUICKAPI_ENSURE_ASCII': QUICKAPI_ENSURE_ASCII,
+        }
     return JSONResponse(data=data)
 
 test.__doc__ = _("""
@@ -119,15 +118,37 @@ Nothing
 #!javascript
 
 {
-    'REMOTE_ADDR': '127.0.0.1' || null,
-    'REMOTE_HOST': 'example.org' || null,
-    'default language': 'en',
-    'request language': 'ru',
-    'string': 'String in your localization',
-    'datetime': '2013-01-01T00:00:00.000Z',
-    'is_authenticated': true,
+    "REMOTE_ADDR": "127.0.0.1" || null,
+    "REMOTE_HOST": "example.org" || null,
+    "default language": "en",
+    "request language": "ru",
+    "is_authenticated": true,
+    "types": {
+        "string": "String in your localization",
+        "datetime": "2014-01-01T00:00:00.000Z",
+        "date": "2014-01-01",
+        "time": "00:00:00.000",
+        "decimal": "12345678.90",
+        "float": 12345678.9,
+        "integer": 1234567890,
+    },
+    "settings": {
+        "QUICKAPI_DEFINED_METHODS": {
+            "quickapi.test": "quickapi.views.test"
+        }, 
+        "QUICKAPI_INDENT": 2, 
+        "QUICKAPI_DECIMAL_LOCALE": false, 
+        "QUICKAPI_ONLY_AUTHORIZED_USERS": false, 
+        "QUICKAPI_DEBUG": false, 
+        "QUICKAPI_SWITCH_LANGUAGE_AUTO": true, 
+        "QUICKAPI_SWITCH_LANGUAGE": true,
+        "QUICKAPI_ENSURE_ASCII": false
+    }
+    
 }
 ```
+
+*In debug mode shows the settings. Here are the default.*
 """)
 
 class Collection(object):
@@ -207,7 +228,7 @@ def get_methods(list_or_dict=QUICKAPI_DEFINED_METHODS, sort=False):
     else:
         raise ValueError('Parameter must be sequence or dictionary')
     for key,val in seq:
-        if isinstance(val, (str, unicode)):
+        if isinstance(val, six.string_types):
             try:
                 method = __import__(val, fromlist=[''])
             except ImportError:
@@ -219,9 +240,9 @@ def get_methods(list_or_dict=QUICKAPI_DEFINED_METHODS, sort=False):
                     method = getattr(module, _method)
                 except ImportError as e:
                     if QUICKAPI_DEBUG:
-                        print colorize(unicode(e), fg='red')
+                        print(colorize(smart_text(e), fg='red'))
                     else:
-                        print e
+                        print(e)
                     method = None
         else:
             method = val
@@ -262,17 +283,17 @@ def index(request, methods=METHODS):
 
     if QUICKAPI_DEBUG:
         p = '\nQUICKAPI:'
-        print colorize(p, fg='blue')
+        print(colorize(p, fg='blue'))
         p = '\trequest.is_ajax()\t== %s' % request.is_ajax()
-        print colorize(p, fg='blue')
+        print(colorize(p, fg='blue'))
 
     if request.is_ajax() or request.method == 'POST':
         try:
             return run(request, methods)
         except Exception as e:
             if QUICKAPI_DEBUG:
-                print colorize(unicode(e), fg='red')
-            return JSONResponse(status=500, message=unicode(e))
+                print(colorize(smart_text(e), fg='red'))
+            return JSONResponse(status=500, message=smart_text(e))
 
     # Vars for docs
     ctx = {}
@@ -289,12 +310,10 @@ api = index
 def run(request, methods):
     """ Авторизует пользователя, если он не авторизован и запускает методы """
 
-    switch_language(request)
-
     is_authenticate = request.user.is_authenticated()
     if QUICKAPI_DEBUG:
         p = '\trun as user\t\t== %s' % request.user
-        print colorize(p, fg='blue')
+        print(colorize(p, fg='blue'))
 
     def _auth(post):
         if request.META.has_key('HTTP_AUTHORIZATION'):
@@ -304,7 +323,11 @@ def run(request, methods):
         else:
             return post.get('username', None), post.get('password', None)
         if key.lower().count('basic'):
-            return key[6:].decode('base64').split(':')
+            if six.PY3:
+                b = bytes(key[6:], settings.DEFAULT_CHARSET).decode('base64_codec')
+            else:
+                b = bytes(key[6:]).decode('base64')
+            return smart_text(b).split(':')
         else:
             return None, None
 
@@ -331,12 +354,12 @@ def run(request, methods):
             pe = '\t%s' % e
             ppost = '\trequest.POST\t\t== %s' % request.POST
             if QUICKAPI_DEBUG:
-                print colorize(pe, fg='red')
-                print colorize(ppost, fg='blue')
+                print(colorize(pe, fg='red'))
+                print(colorize(ppost, fg='blue'))
             else:
-                print pe
-                print ppost
-            return JSONResponse(status=400, message=unicode(e))
+                print(pe)
+                print(ppost)
+            return JSONResponse(status=400, message=smart_text(e))
         else:
             if not is_authenticate:
                 username, password = _auth(json)
@@ -352,17 +375,17 @@ def run(request, methods):
 
     if QUICKAPI_DEBUG:
         p = '\tlogin user\t\t== %s' % request.user
-        print colorize(p, fg='blue')
+        print(colorize(p, fg='blue'))
         p = '\tmethod\t\t\t== %s' % method
-        print colorize(p, fg='blue')
+        print(colorize(p, fg='blue'))
 
     if method in methods:
         try:
             real_method = methods[method]['method']
         except Exception as e:
-            msg = traceback.format_exc(e)
+            msg = traceback.format_exc()
             if DEBUG:
-                print msg
+                print(msg)
             else:
                 msg = MESSAGES[405]
             return JSONResponse(status=405, message=msg)
@@ -371,26 +394,26 @@ def run(request, methods):
     else:
         msg = _('Method `%s` does not exist') % method
         if DEBUG:
-            print msg
+            print(msg)
         return JSONResponse(status=405, message=msg)
 
     try:
         return real_method(request, **kwargs)
     except Exception as e:
-        msg = traceback.format_exc(e)
+        msg = traceback.format_exc()
         if DEBUG:
-            print msg
+            print(msg)
         else:
             try:
                 msg = msg.decode('utf-8')
             except:
                 pass
-            mail_admins(u'QuickAPI method error', msg +'\n\n'+ unicode(request))
+            mail_admins('QuickAPI method error', msg +'\n\n'+ smart_text(request))
         try:
             return JSONResponse(status=500, message=str(e).decode('utf-8'))
         except:
             try:
-                return JSONResponse(status=500, message=unicode(e))
+                return JSONResponse(status=500, message=smart_text(e))
             except:
                 pass
             return JSONResponse(status=500, message=MESSAGES[500])
